package com.jpattern.gwt.client;

import java.util.HashMap;
import java.util.Map;

import com.jpattern.gwt.client.cache.CacheService;
import com.jpattern.gwt.client.cache.ICacheService;
import com.jpattern.gwt.client.history.DirectHistoryEngine;
import com.jpattern.gwt.client.history.HistoryService;
import com.jpattern.gwt.client.history.IHistoryService;
import com.jpattern.gwt.client.history.NullHistoryManagerObserver;
import com.jpattern.gwt.client.logger.ILoggerService;
import com.jpattern.gwt.client.logger.NullLoggerService;
import com.jpattern.gwt.client.security.ISecurityContext;
import com.jpattern.gwt.client.security.SecurityContext;
import com.jpattern.gwt.client.serializer.ISerializerService;
import com.jpattern.gwt.client.serializer.NullSerializerService;

/**
 * 
 * @author Francesco Cina'
 *
 * 12 Apr 2011
 */
public abstract class AApplication implements IApplicationProvider, IApplicationSystem {

	private ISerializerService serializerService = new NullSerializerService();
	private ILoggerService loggerService = new NullLoggerService();
	private IServerCallService serverCallService = new RestServerCallService();
	private Map<String, IService> serviceMap = new HashMap<String, IService>();
	private ICacheService cacheService = new CacheService();
	private IHistoryService historyService = new HistoryService(new DirectHistoryEngine(new NullLoggerService()), new NullHistoryManagerObserver(), new NullLoggerService());
	private IManagedErrorService managedErrorService = new ManagedErrorService();
	private ISecurityContext securityContext = new SecurityContext();

	@Override
	public final ILoggerService getLoggerService() {
		return loggerService;
	}
	
	@Override
	public final void setLoggerService(ILoggerService loggerService) {
		if (loggerService!=null) {
			this.loggerService = loggerService;
		}
	}
	
	@Override
	public final IServerCallService getServerCallService() {
		return serverCallService;
	}
	
	@Override
	public final void setServerCallService(IServerCallService serverCallService) {
		if (serverCallService!=null) {
			this.serverCallService = serverCallService;
		}
	}
	
	@Override
	public final IService getService(String serviceName) {
		if (serviceMap.containsKey(serviceName)) {
			return serviceMap.get(serviceName);
		}
		return new NullService();
	}
	
	@Override
	public final void addService(String serviceName, IService service){
		serviceMap.put(serviceName, service);
	}

	@Override
	public final ISerializerService getSerializerService() {
		return serializerService;
	}

	@Override
	public final void setSerializerService(ISerializerService serializerService) {
		this.serializerService = serializerService;
	}
	
	@Override
	public final ICacheService getCacheService() {
		return cacheService ;
	}
	
	@Override
	public final IHistoryService getHistoryService() {
		return historyService;
	}

	@Override
	public final void setHistoryService(IHistoryService historyService) {
		this.historyService = historyService;
	}
	
	@Override
	public final IManagedErrorService getManagedErrorService() {
		return managedErrorService;
	}
	
	@Override
	public final ISecurityContext getSecurityContext() {
		return securityContext;
	}
	
	@Override
	public void setSecurityContext(ISecurityContext securityContext) {
		this.securityContext = securityContext;
	}

}
